# -*- coding: latin-1 -*-
#########################################################
#Author: Jean-Baptiste Fasquel, ISTIA, Angers University, France
#Copyright (C) 2014 Jean-Baptiste Fasquel
#########################################################

import cv2
import numpy as np
import helper

##################################
#REFERENCE IMAGE
image_ref = cv2.imread('reference_image_logo.png')
mask=cv2.imread('reference_image_logo_mask.png',0) #0: pour un channel
detector = cv2.xfeatures2d.SURF_create()
kp_ref, ds_ref = detector.detectAndCompute(image_ref,mask)
##################################
#INITIAL POSITION OF THE VIRTUAL OBJECT 
corners=helper.keypoints2numpyarray(kp_ref)
K=helper.create_K()
corners_3d=helper.pixels2camera(corners,K,Z=500)
square_3d=helper.create_square(corners_3d,size=60)
square_3d_h=helper.coord2homogeneous(square_3d)

##################################
#NEW POSE
for i in range(1,14): 
#for i in range(10,11):
    #Current image
    image_current = cv2.imread('pose_logo_'+str(i)+'.png')
    #image_current = cv2.imread('reference_image_logo.png')
    #DESCRIPTORS
    keypoints, descriptors = detector.detectAndCompute(image_current,None)
    #MATCHING
    bf = cv2.BFMatcher(cv2.NORM_L2, crossCheck=True)
    matches = bf.match(descriptors,ds_ref)
    matches = sorted(matches, key = lambda x:x.distance)
    #SUBSET OF MATCHING KEYPOINTS (FROM REF AND CURRENT)
    kp_ref_subpart=[]
    kp_current=[]
    for m in matches[0:10]:
        tmp_kp=kp_ref[m.trainIdx]
        kp_ref_subpart+=[tmp_kp]
        tmp_kp=keypoints[m.queryIdx]
        kp_current+=[tmp_kp]
    corners_2d=helper.keypoints2numpyarray(kp_ref_subpart)
    corners_3d=helper.pixels2camera(corners_2d,K,Z=500)

    #Homography
    refpt=helper.keypoints2numpyarray(kp_ref_subpart)
    currentpt=helper.keypoints2numpyarray(kp_current)
    H = cv2.findHomography( refpt, currentpt, cv2.RANSAC,ransacReprojThreshold=3 )[0]
    print("NEW H",H)
    nb,rvec,tvec,norms=cv2.decomposeHomographyMat(H,K)
    M_ext_h=np.hstack((rvec[0],tvec[0]))
    P_h=np.dot(K,M_ext_h)
    registered_square_ransac=helper.world2pixels(square_3d_h,P_h)
    #print(M_ext_ransac)
    #print(M_ext_h)
    #print("NEW RVEC",rvec);print("NEW TVEC",tvec);print("Norm",norms)
    #quit()
    #DISPLAY
    image_current=cv2.drawKeypoints(image_current,kp_current,image_current)    
    #cv2.polylines(image_current, [np.int32(registered_square)], True, (0, 0, 255), 4)
    cv2.polylines(image_current, [np.int32(registered_square_ransac)], True, (255, 0, 0), 4)
    cv2.imshow('Ransac (blue), no ransac (red)',image_current)
    cv2.imwrite('registration_'+str(i)+'.png',image_current)
    cv2.waitKey()
